@@ -19,10 +19,11 @@ use File::Path;
 use strict;
 use vars qw( %TEST_SCRIPTS $VERSION );
 
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 # ----------------------------------------------------------------------------
 
+# See below for descriptions
 %TEST_SCRIPTS = (
   1 => [<<'EOF',
 use lib './blib/lib';
@@ -99,6 +100,22 @@ print "Test uncached output 5\n";
 EOF
          ["Test output 5\n","Test output 5\nTest uncached output 5\n"]],
 
+  6 => [<<'EOF',
+use lib './blib/lib';
+use CGI::Cache;
+
+$SIG{__DIE__} = sub { print STDOUT @_;exit 1 };
+
+CGI::Cache::setup( );
+CGI::Cache::set_key( {2 => 'test key 3', 1 => 'test', 3 => 'key'} );
+CGI::Cache::start();
+
+print "Test output 6\n";
+
+die "STDERR!" if @ARGV;
+EOF
+         "Test output 6\n"],
+
 );
 
 # ----------------------------------------------------------------------------
@@ -111,6 +128,7 @@ print "1..$total\n";
 my $test_number = 1;
 
 foreach my $test (@$tests) {
+#$test_number++ and next unless $test_number =~ /^(1|2|11)$/;
 	if (&$test) {
 		print "ok $test_number\n";
 	} else {
@@ -125,7 +143,7 @@ foreach my $test (@$tests) {
 sub get_tests {
 	[
 
-# Test that module can be loaded without errors
+# Test 1: that the module can be loaded without errors
 sub {
 	unless (eval "require CGI::Cache")
   {
@@ -136,7 +154,7 @@ sub {
 	1;
 },
 
-# Test that we can initialize the cache with the default values
+# Test 2: that we can initialize the cache with the default values
 sub {
   my $x;
 	$@ = '';
@@ -148,7 +166,7 @@ sub {
 	(($x == 1) && ($@ eq '')) ? 1 : 0;
 },
 
-# Test that we can initialize the cache with the non-default values
+# Test 3: that we can initialize the cache with the non-default values
 sub {
   my $x;
   $@ = '';
@@ -165,7 +183,7 @@ sub {
 	(($x == 1) && ($@ eq '')) ? 1 : 0;
 },
 
-# Test that we can set a simple key
+# Test 4: that we can set a simple key
 sub {
   my $x;
   $@ = '';
@@ -177,7 +195,7 @@ sub {
 	(($x == 1) && ($@ eq '')) ? 1 : 0;
 },
 
-# Test that we can set a complex key
+# Test 5: that we can set a complex key
 sub {
   my $x;
   $@ = '';
@@ -189,17 +207,17 @@ sub {
 	(($x == 1) && ($@ eq '')) ? 1 : 0;
 },
 
-# Test caching with default attributes
+# Test 6: caching with default attributes
 sub {
 	return time_script(1);
 },
 
-# Test caching with some custom attributes, and with a complex data structure
+# Test 7: caching with some custom attributes, and with a complex data structure
 sub {
 	return time_script(2);
 },
 
-# Test that a script with an error doesn't cache output
+# Test 8: that a script with an error doesn't cache output
 sub {
   my $script_number = 3;
 
@@ -223,8 +241,8 @@ sub {
     open STDERR,">&OLDSTDERR" or die "Can't restore STDERR: $!\n";
 
     # Make sure there is nothing in the cache
-		(!defined $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY)) ||
-			die "A script with an error resulted in cached content.";
+		die "A script with an error resulted in cached content."
+		  if defined $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY);
 
 		#	Now run successfully and make sure the cached content is there
 		my $script_results_2 = `perl $file`;
@@ -232,11 +250,11 @@ sub {
     # Get the real answer and compare that to what test generated
     my $real_results = $TEST_SCRIPTS{$script_number}[1];
 
-		($real_results eq $script_results_2) ||
-			die "Second run of test script didn't return the correct content.";
+		die "Second run of test script didn't return the correct content."
+		 if $real_results ne $script_results_2;
 
-		($real_results eq $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY)) ||
-			die "Cache file didn't have right content.";
+		die "Cache file didn't have right content."
+		 if $real_results ne $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY);
 	};
 
 	unlink "$file";
@@ -245,7 +263,7 @@ sub {
 	($@ eq '') ? 1 : 0;
 },
 
-# Test that a script that prints to STDERR doesn't cache output
+# Test 9: that a script that prints to STDERR doesn't cache output
 sub {
   my $script_number = 4;
 
@@ -269,8 +287,8 @@ sub {
     open STDERR,">&OLDSTDERR" or die "Can't restore STDERR: $!\n";
 
     # Make sure there is nothing in the cache
-		(!defined $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY)) ||
-			die "A script with output to STDERR resulted in cached content.";
+		die "A script with output to STDERR resulted in cached content."
+		  if defined $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY);
 
 		#	Now run successfully and make sure the cached content is there
 		my $script_results_2 = `perl $file`;
@@ -278,11 +296,11 @@ sub {
     # Get the real answer and compare that to what test generated
     my $real_results = $TEST_SCRIPTS{$script_number}[1];
 
-		($real_results eq $script_results_2) ||
-			die "Second run of test script didn't return the correct content.";
+		die "Second run of test script didn't return the correct content."
+		  if $real_results ne $script_results_2;
 
-		($real_results eq $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY)) ||
-			die "Cache file didn't have right content.";
+		die "Cache file didn't have right content."
+		  if $real_results ne $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY);
 	};
 
 	unlink "$file";
@@ -291,7 +309,7 @@ sub {
 	($@ eq '') ? 1 : 0;
 },
 
-# Test that stop() actually stops caching output
+# Test 10: that stop() actually stops caching output
 sub {
   my $script_number = 5;
 
@@ -308,14 +326,14 @@ sub {
     # Get the real STDOUT and compare that to what test generated
     my $real_output_results = $TEST_SCRIPTS{$script_number}[1][1];
 
-		($real_output_results eq $script_output) ||
-			die "Test script didn't output the correct content to STDOUT.";
+		die "Test script didn't output the correct content to STDOUT."
+		  if $real_output_results ne $script_output;
 
     # Get the real cached data and compare that to what test generated
     my $real_cache_results = $TEST_SCRIPTS{$script_number}[1][0];
 
-		($real_cache_results eq $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY)) ||
-			die "Cache file didn't have right content.";
+		die "Cache file didn't have right content."
+		  if $real_cache_results ne $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY);
 	};
 
 	unlink "$file";
@@ -323,6 +341,49 @@ sub {
 	($@ eq '') ? 1 : 0;
 },
 
+# Test 11: that a script that calls a redirected die doesn't cache output
+sub {
+  my $script_number = 6;
+
+	my $file = "cgi_test_$script_number.cgi";
+
+  write_script($script_number,$file);
+  setup_cache($script_number,$file);
+
+	$@ = '';
+	eval {
+    # Save STDERR and redirect temporarily to nothing. This will prevent the
+    # test script from emitting output to STDERR
+    use vars qw(*OLDSTDERR);
+    open OLDSTDERR,">&STDERR" or die "Can't save STDERR: $!\n";
+    open STDERR,">STDERR-redirected"
+      or die "Can't redirect STDERR to STDERR-redirected: $!\n";
+
+		#	First run should die after printing some output
+		`perl $file args`;
+
+    open STDERR,">&OLDSTDERR" or die "Can't restore STDERR: $!\n";
+
+    # Make sure there is nothing in the cache
+		die "A script that called a redirected die() resulted in cached content."
+		  if defined $CGI::Cache::CACHE->get($CGI::Cache::CACHE_KEY);
+
+		#	Now run successfully and make sure the cached content is there
+		my $script_results_2 = `perl $file`;
+
+    # Get the real answer and compare that to what test generated
+    my $real_results = $TEST_SCRIPTS{$script_number}[1];
+
+		die "Second run of test script didn't return the correct content."
+		  if $real_results ne $script_results_2;
+	};
+
+	unlink "$file";
+	unlink "STDERR-redirected";
+
+	($@ eq '') ? 1 : 0;
+},
+
 	];
 }
 
@@ -405,6 +466,7 @@ sub setup_cache
   my ($cache_key) = $TEST_SCRIPTS{$script_number}[0] =~ /set_key\((.*?)\)/s;
 
   $ENV{SCRIPT_NAME} = $file;
+
   eval "CGI::Cache::setup($cache_options)";
   eval "CGI::Cache::set_key($cache_key)";
 
